@@ -25,10 +25,8 @@
 #include <QDebug>
 #include <klocalizedstring.h>
 
-#include <QFileInfo>
 #include <QEventLoop>
 #include <QDir>
-#include <QVariant>
 #include <QUrl>
 
 #include <kprotocolinfo.h>
@@ -121,8 +119,8 @@ void JobRemoteTest::cleanupTestCase()
 void JobRemoteTest::enterLoop()
 {
     QEventLoop eventLoop;
-    connect(this, SIGNAL(exitLoop()),
-            &eventLoop, SLOT(quit()));
+    connect(this, &JobRemoteTest::exitLoop,
+            &eventLoop, &QEventLoop::quit);
     eventLoop.exec(QEventLoop::ExcludeUserInputEvents);
 }
 
@@ -133,14 +131,14 @@ void JobRemoteTest::putAndGet()
     QUrl u(remoteTmpUrl());
     u.setPath(u.path() + "putAndGetFile");
     KIO::TransferJob *job = KIO::put(u, 0600, KIO::Overwrite | KIO::HideProgressInfo);
-    QDateTime mtime = QDateTime::currentDateTime().addSecs(-30);   // 30 seconds ago
-    mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
+    quint64 secsSinceEpoch = QDateTime::currentSecsSinceEpoch(); // Use second granularity, supported on all filesystems
+    QDateTime mtime = QDateTime::fromSecsSinceEpoch(secsSinceEpoch - 30); // 30 seconds ago
     job->setModificationTime(mtime);
     job->setUiDelegate(nullptr);
-    connect(job, SIGNAL(result(KJob*)),
-            this, SLOT(slotResult(KJob*)));
-    connect(job, SIGNAL(dataReq(KIO::Job*,QByteArray&)),
-            this, SLOT(slotDataReq(KIO::Job*,QByteArray&)));
+    connect(job, &KJob::result,
+            this, &JobRemoteTest::slotResult);
+    connect(job, &KIO::TransferJob::dataReq,
+            this, &JobRemoteTest::slotDataReq);
     m_result = -1;
     m_dataReqCount = 0;
     enterLoop();
@@ -150,8 +148,8 @@ void JobRemoteTest::putAndGet()
 
     KIO::StoredTransferJob *getJob = KIO::storedGet(u, KIO::NoReload, KIO::HideProgressInfo);
     getJob->setUiDelegate(nullptr);
-    connect(getJob, SIGNAL(result(KJob*)),
-            this, SLOT(slotGetResult(KJob*)));
+    connect(getJob, &KJob::result,
+            this, &JobRemoteTest::slotGetResult);
     enterLoop();
     QCOMPARE(m_result, 0);   // no error
     QCOMPARE(m_data, QByteArray("This is a test for KIO::put()\n"));
@@ -198,19 +196,20 @@ void JobRemoteTest::openFileWriting()
     fileJob = KIO::open(u, QIODevice::WriteOnly);
 
     fileJob->setUiDelegate(nullptr);
-    connect(fileJob, SIGNAL(result(KJob*)),
-            this, SLOT(slotResult(KJob*)));
-    connect(fileJob, SIGNAL(data(KIO::Job*,QByteArray)),
-            this, SLOT(slotFileJobData(KIO::Job*,QByteArray)));
-    connect(fileJob, SIGNAL(open(KIO::Job*)),
-            this, SLOT(slotFileJobOpen(KIO::Job*)));
-    connect(fileJob, SIGNAL(written(KIO::Job*,KIO::filesize_t)),
-            this, SLOT(slotFileJobWritten(KIO::Job*,KIO::filesize_t)));
-    connect(fileJob, SIGNAL(position(KIO::Job*,KIO::filesize_t)),
-            this, SLOT(slotFileJobPosition(KIO::Job*,KIO::filesize_t)));
-    connect(fileJob, SIGNAL(close(KIO::Job*)),
-            this, SLOT(slotFileJobClose(KIO::Job*)));
+    connect(fileJob, &KJob::result,
+            this, &JobRemoteTest::slotResult);
+    connect(fileJob, &KIO::FileJob::data,
+            this, &JobRemoteTest::slotFileJobData);
+    connect(fileJob, &KIO::FileJob::open,
+            this, &JobRemoteTest::slotFileJobOpen);
+    connect(fileJob, &KIO::FileJob::written,
+            this, &JobRemoteTest::slotFileJobWritten);
+    connect(fileJob, &KIO::FileJob::position,
+            this, &JobRemoteTest::slotFileJobPosition);
+    connect(fileJob, QOverload<KIO::Job*>::of(&KIO::FileJob::close),
+            this, &JobRemoteTest::slotFileJobClose);
     m_result = -1;
+    m_closeSignalCalled = false;
 
     enterLoop();
     QEXPECT_FAIL("", "Needs fixing in kio_file", Abort);
@@ -218,10 +217,11 @@ void JobRemoteTest::openFileWriting()
 
     KIO::StoredTransferJob *getJob = KIO::storedGet(u, KIO::NoReload, KIO::HideProgressInfo);
     getJob->setUiDelegate(nullptr);
-    connect(getJob, SIGNAL(result(KJob*)),
-            this, SLOT(slotGetResult(KJob*)));
+    connect(getJob, &KJob::result,
+            this, &JobRemoteTest::slotGetResult);
     enterLoop();
     QCOMPARE(m_result, 0);   // no error
+    QVERIFY(m_closeSignalCalled); // close signal called.
     qDebug() << "m_data: " << m_data;
     QCOMPARE(m_data, QByteArray("test....test....test....test....test....test....end"));
 
@@ -275,6 +275,7 @@ void JobRemoteTest::slotFileJobPosition(KIO::Job *job, KIO::filesize_t offset)
 void JobRemoteTest::slotFileJobClose(KIO::Job *job)
 {
     Q_UNUSED(job);
+    m_closeSignalCalled = true;
     qDebug() << "+++++++++ closed";
 }
 
@@ -292,12 +293,12 @@ void JobRemoteTest::openFileReading()
                                      0600, KIO::Overwrite | KIO::HideProgressInfo
                                                    );
 
-    QDateTime mtime = QDateTime::currentDateTime().addSecs(-30);   // 30 seconds ago
-    mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
+    quint64 secsSinceEpoch = QDateTime::currentSecsSinceEpoch(); // Use second granularity, supported on all filesystems
+    QDateTime mtime = QDateTime::fromSecsSinceEpoch(secsSinceEpoch - 30); // 30 seconds ago
     putJob->setModificationTime(mtime);
     putJob->setUiDelegate(nullptr);
-    connect(putJob, SIGNAL(result(KJob*)),
-            this, SLOT(slotResult(KJob*)));
+    connect(putJob, &KJob::result,
+            this, &JobRemoteTest::slotResult);
     m_result = -1;
     enterLoop();
     QVERIFY(m_result == 0);   // no error
@@ -308,22 +309,24 @@ void JobRemoteTest::openFileReading()
     fileJob = KIO::open(u, QIODevice::ReadOnly);
 
     fileJob->setUiDelegate(nullptr);
-    connect(fileJob, SIGNAL(result(KJob*)),
-            this, SLOT(slotResult(KJob*)));
-    connect(fileJob, SIGNAL(data(KIO::Job*,QByteArray)),
-            this, SLOT(slotFileJob2Data(KIO::Job*,QByteArray)));
-    connect(fileJob, SIGNAL(open(KIO::Job*)),
-            this, SLOT(slotFileJob2Open(KIO::Job*)));
-    connect(fileJob, SIGNAL(written(KIO::Job*,KIO::filesize_t)),
-            this, SLOT(slotFileJob2Written(KIO::Job*,KIO::filesize_t)));
-    connect(fileJob, SIGNAL(position(KIO::Job*,KIO::filesize_t)),
-            this, SLOT(slotFileJob2Position(KIO::Job*,KIO::filesize_t)));
-    connect(fileJob, SIGNAL(close(KIO::Job*)),
-            this, SLOT(slotFileJob2Close(KIO::Job*)));
+    connect(fileJob, &KJob::result,
+            this, &JobRemoteTest::slotResult);
+    connect(fileJob, &KIO::FileJob::data,
+            this, &JobRemoteTest::slotFileJob2Data);
+    connect(fileJob, &KIO::FileJob::open,
+            this, &JobRemoteTest::slotFileJob2Open);
+    connect(fileJob, &KIO::FileJob::written,
+            this, &JobRemoteTest::slotFileJob2Written);
+    connect(fileJob, &KIO::FileJob::position,
+            this, &JobRemoteTest::slotFileJob2Position);
+    connect(fileJob, QOverload<KIO::Job*>::of(&KIO::FileJob::close),
+            this, &JobRemoteTest::slotFileJob2Close);
     m_result = -1;
+    m_closeSignalCalled = false;
 
     enterLoop();
     QVERIFY(m_result == 0);   // no error
+    QVERIFY(m_closeSignalCalled); // close signal called.
     qDebug() << "resulting m_data: " << QString(m_data);
     QCOMPARE(m_data, QByteArray("test5test4test3test2test1"));
 
@@ -376,6 +379,7 @@ void JobRemoteTest::slotFileJob2Position(KIO::Job *job, KIO::filesize_t offset)
 void JobRemoteTest::slotFileJob2Close(KIO::Job *job)
 {
     Q_UNUSED(job);
+    m_closeSignalCalled = true;
     qDebug() << "+++++++++ job2 closed";
 }
 
@@ -387,3 +391,79 @@ void JobRemoteTest::slotMimetype(KIO::Job *job, const QString &type)
     m_mimetype = type;
 }
 
+void JobRemoteTest::openFileRead0Bytes()
+{
+    QUrl u(remoteTmpUrl());
+    u.setPath(u.path() + "openFileReading");
+
+    const QByteArray putData("Doesn't matter");
+
+    KIO::StoredTransferJob *putJob = KIO::storedPut(putData,
+                                     u,
+                                     0600, KIO::Overwrite | KIO::HideProgressInfo
+                                                   );
+
+    quint64 secsSinceEpoch = QDateTime::currentSecsSinceEpoch(); // Use second granularity, supported on all filesystems
+    QDateTime mtime = QDateTime::fromSecsSinceEpoch(secsSinceEpoch - 30); // 30 seconds ago
+    putJob->setModificationTime(mtime);
+    putJob->setUiDelegate(nullptr);
+    connect(putJob, &KJob::result,
+            this, &JobRemoteTest::slotResult);
+    m_result = -1;
+    enterLoop();
+    QVERIFY(m_result == 0);   // no error
+
+    m_data = QByteArray();
+
+    fileJob = KIO::open(u, QIODevice::ReadOnly);
+
+    fileJob->setUiDelegate(nullptr);
+    connect(fileJob, &KJob::result,
+            this, &JobRemoteTest::slotResult);
+    connect(fileJob, &KIO::FileJob::data,
+            this, &JobRemoteTest::slotFileJob3Data);
+    connect(fileJob, &KIO::FileJob::open,
+            this, &JobRemoteTest::slotFileJob3Open);
+    // Can reuse this slot (it's a noop).
+    connect(fileJob, &KIO::FileJob::written,
+            this, &JobRemoteTest::slotFileJob2Written);
+    connect(fileJob, &KIO::FileJob::position,
+            this, &JobRemoteTest::slotFileJob3Position);
+    // Can reuse this as well.
+    connect(fileJob, QOverload<KIO::Job*>::of(&KIO::FileJob::close),
+            this, &JobRemoteTest::slotFileJob3Close);
+    m_result = -1;
+    m_closeSignalCalled = false;
+
+    enterLoop();
+    // Previously reading 0 bytes would cause both data() and error() being emitted...
+    QVERIFY(m_result == 0);   // no error
+    QVERIFY(m_closeSignalCalled); // close signal called.
+}
+
+void JobRemoteTest::slotFileJob3Open(KIO::Job *job)
+{
+    Q_UNUSED(job);
+    fileJob->seek(0);
+}
+
+void JobRemoteTest::slotFileJob3Position(KIO::Job *job, KIO::filesize_t offset)
+{
+    Q_UNUSED(job);
+    qDebug() << "position : " << offset << " -> read (0)";
+    fileJob->read(0);
+}
+
+void JobRemoteTest::slotFileJob3Data(KIO::Job *job, const QByteArray& data)
+{
+    Q_UNUSED(job);
+    QVERIFY(data.isEmpty());
+    fileJob->close();
+}
+
+void JobRemoteTest::slotFileJob3Close(KIO::Job *job)
+{
+    Q_UNUSED(job);
+    m_closeSignalCalled = true;
+    qDebug() << "+++++++++ job2 closed";
+}
