@@ -23,19 +23,22 @@
 #include "ksambasharedata_p.h"
 #include "kiocoredebug.h"
 
-#include <QtCore/QMap>
-#include <QtCore/QMutableMapIterator>
-#include <QtCore/QFile>
-#include <QtCore/QRegExp>
-#include <QtCore/QFileInfo>
-#include <QtCore/QTextStream>
-#include <QtCore/QStringList>
-#include <QtCore/QProcess>
+#include <QMap>
+#include <QFile>
+#include <QLoggingCategory>
+#include <QRegExp>
+#include <QFileInfo>
+#include <QTextStream>
+#include <QStringList>
+#include <QProcess>
 #include <QDebug>
 
 #include <kdirwatch.h>
 #include <kuser.h>
 
+Q_DECLARE_LOGGING_CATEGORY(KIO_CORE_SAMBASHARE)
+Q_LOGGING_CATEGORY(KIO_CORE_SAMBASHARE, "kf5.kio.core.sambashare", QtWarningMsg)
+
 // Default smb.conf locations
 // sorted by priority, most priority first
 static const char *const DefaultSambaConfigFilePathList[] = {
@@ -59,7 +62,7 @@ KSambaSharePrivate::KSambaSharePrivate(KSambaShare *parent)
 {
     setUserSharePath();
     findSmbConf();
-    sync();
+    data = parse(getNetUserShareInfo());
 }
 
 KSambaSharePrivate::~KSambaSharePrivate()
@@ -83,14 +86,14 @@ bool KSambaSharePrivate::isSambaInstalled()
 bool KSambaSharePrivate::findSmbConf()
 {
     for (int i = 0; i < DefaultSambaConfigFilePathListSize; ++i) {
-        const QString filePath(DefaultSambaConfigFilePathList[i]);
+        const QString filePath = QString::fromLatin1(DefaultSambaConfigFilePathList[i]);
         if (QFile::exists(filePath)) {
             smbConf = filePath;
             return true;
         }
     }
 
-    qCWarning(KIO_CORE) << "KSambaShare: Could not find smb.conf!";
+    qCDebug(KIO_CORE_SAMBASHARE) << "KSambaShare: Could not find smb.conf!";
 
     return false;
 }
@@ -125,12 +128,15 @@ QString KSambaSharePrivate::testparmParamValue(const QString &parameterName)
         return QString();
     }
 
-    QStringList args;
     QByteArray stdErr;
     QByteArray stdOut;
 
-    args << QStringLiteral("-d0") << QStringLiteral("-s") << QStringLiteral("--parameter-name")
-         << parameterName;
+    const QStringList args{
+        QStringLiteral("-d0"),
+        QStringLiteral("-s"),
+        QStringLiteral("--parameter-name"),
+        parameterName,
+    };
 
     runProcess(QStringLiteral("testparm"), args, stdOut, stdErr);
 
@@ -162,11 +168,13 @@ QByteArray KSambaSharePrivate::getNetUserShareInfo()
         return QByteArray();
     }
 
-    QStringList args;
     QByteArray stdOut;
     QByteArray stdErr;
 
-    args << QStringLiteral("usershare") << QStringLiteral("info");
+    const QStringList args{
+        QStringLiteral("usershare"),
+        QStringLiteral("info"),
+    };
 
     runProcess(QStringLiteral("net"), args, stdOut, stdErr);
 
@@ -281,7 +289,7 @@ KSambaShareData::UserShareError KSambaSharePrivate::isPathValid(const QString &p
 
 KSambaShareData::UserShareError KSambaSharePrivate::isAclValid(const QString &acl) const
 {
-    const QRegExp aclRx(QStringLiteral("(?:(?:(\\w+\\s*)\\\\|)(\\w+\\s*):([fFrRd]{1})(?:,|))*"));
+    const QRegExp aclRx(QStringLiteral("(?:(?:(\\w(\\w|\\s)*)\\\\|)(\\w+\\s*):([fFrRd]{1})(?:,|))*"));
     // TODO: check if user is a valid smb user
     return aclRx.exactMatch(acl) ? KSambaShareData::UserShareAclOk
            : KSambaShareData::UserShareAclInvalid;
@@ -309,7 +317,6 @@ KSambaShareData::UserShareError KSambaSharePrivate::add(const KSambaShareData &s
         return KSambaShareData::UserShareSystemError;
     }
 
-    QStringList args;
     QByteArray stdOut;
     QByteArray stdErr;
 
@@ -327,8 +334,15 @@ KSambaShareData::UserShareError KSambaSharePrivate::add(const KSambaShareData &s
                           (shareData.guestPermission() == KSambaShareData::GuestsNotAllowed)
                           ? QStringLiteral("n") : QStringLiteral("y"));
 
-    args << QStringLiteral("usershare") << QStringLiteral("add") << shareData.name()
-         << shareData.path() << shareData.comment() << shareData.acl() << guestok;
+    const QStringList args{
+        QStringLiteral("usershare"),
+        QStringLiteral("add"),
+        shareData.name(),
+        shareData.path(),
+        shareData.comment(),
+        shareData.acl(),
+        guestok,
+    };
 
     int ret = runProcess(QStringLiteral("net"), args, stdOut, stdErr);
 
@@ -349,19 +363,22 @@ KSambaShareData::UserShareError KSambaSharePrivate::remove(const KSambaShareData
         return KSambaShareData::UserShareSystemError;
     }
 
-    QStringList args;
 
     if (!data.contains(shareData.name())) {
         return KSambaShareData::UserShareNameInvalid;
     }
 
-    args << QStringLiteral("usershare") << QStringLiteral("delete") << shareData.name();
+    const QStringList args{
+        QStringLiteral("usershare"),
+        QStringLiteral("delete"),
+        shareData.name(),
+    };
 
     int result = QProcess::execute(QStringLiteral("net"), args);
     return (result == 0) ? KSambaShareData::UserShareOk : KSambaShareData::UserShareSystemError;
 }
 
-bool KSambaSharePrivate::sync()
+QMap<QString, KSambaShareData> KSambaSharePrivate::parse(const QByteArray &usershareData)
 {
     const QRegExp headerRx(QString::fromLatin1("^\\s*\\["
                                          "([^%<>*\?|/\\+=;:\",]+)"
@@ -371,29 +388,34 @@ bool KSambaSharePrivate::sync()
                                          "="
                                          "(.*)$"));
 
-    QTextStream stream(getNetUserShareInfo());
+    QTextStream stream(usershareData);
     QString currentShare;
-    QStringList shareList;
+    QMap<QString, KSambaShareData> shares;
 
     while (!stream.atEnd()) {
         const QString line = stream.readLine().trimmed();
 
         if (headerRx.exactMatch(line)) {
             currentShare = headerRx.cap(1).trimmed();
-            shareList << currentShare;
 
-            if (!data.contains(currentShare)) {
+            if (!shares.contains(currentShare)) {
                 KSambaShareData shareData;
                 shareData.dd->name = currentShare;
-                data.insert(currentShare, shareData);
+                shares.insert(currentShare, shareData);
             }
         } else if (OptValRx.exactMatch(line)) {
             const QString key = OptValRx.cap(1).trimmed();
             const QString value = OptValRx.cap(2).trimmed();
-            KSambaShareData shareData = getShareByName(currentShare);
+            KSambaShareData shareData = shares[currentShare];
 
             if (key == QLatin1String("path")) {
-                shareData.dd->path = value;
+                // Samba accepts paths with and w/o trailing slash, we
+                // use and expect path without slash
+                if (value.endsWith(QLatin1Char('/'))) {
+                    shareData.dd->path = value.left(value.size() - 1);
+                } else {
+                    shareData.dd->path = value;
+                }
             } else if (key == QLatin1String("comment")) {
                 shareData.dd->comment = value;
             } else if (key == QLatin1String("usershare_acl")) {
@@ -407,24 +429,19 @@ bool KSambaSharePrivate::sync()
         } else if (line.trimmed().isEmpty()) {
             continue;
         } else {
-            return false;
+            return shares;
         }
     }
 
-    QMutableMapIterator<QString, KSambaShareData> i(data);
-    while (i.hasNext()) {
-        i.next();
-        if (!shareList.contains(i.key())) {
-            i.remove();
-        }
-    }
-
-    return true;
+    return shares;
 }
 
-void KSambaSharePrivate::_k_slotFileChange(const QString &/*path*/)
+void KSambaSharePrivate::_k_slotFileChange(const QString &path)
 {
-    sync();
+    if (path != userSharePath) {
+        return;
+    }
+    data = parse(getNetUserShareInfo());
     //qDebug() << "path changed:" << path;
     Q_Q(KSambaShare);
     emit q->changed();
@@ -435,7 +452,7 @@ KSambaShare::KSambaShare()
     , d_ptr(new KSambaSharePrivate(this))
 {
     Q_D(const KSambaShare);
-    if (QFile::exists(d->userSharePath)) {
+    if (!d->userSharePath.isEmpty() && QFileInfo::exists(d->userSharePath)) {
         KDirWatch::self()->addDir(d->userSharePath, KDirWatch::WatchFiles);
         connect(KDirWatch::self(), SIGNAL(dirty(QString)), this,
                 SLOT(_k_slotFileChange(QString)));
