@@ -30,12 +30,8 @@
 #include <time.h>
 
 #include <QDebug>
-
-#include <QDBusConnection>
-#include <QPointer>
+#include <QDateTime>
 #include <QDataStream>
-#include <QSslCertificate>
-#include <QSslError>
 
 using namespace KIO;
 
@@ -44,7 +40,7 @@ Q_GLOBAL_STATIC(UserNotificationHandler, globalUserNotificationHandler)
 SlaveInterface::SlaveInterface(SlaveInterfacePrivate &dd, QObject *parent)
     : QObject(parent), d_ptr(&dd)
 {
-    connect(&d_ptr->speed_timer, SIGNAL(timeout()), SLOT(calcSpeed()));
+    connect(&d_ptr->speed_timer, &QTimer::timeout, this, &SlaveInterface::calcSpeed);
 }
 
 SlaveInterface::~SlaveInterface()
@@ -92,7 +88,7 @@ bool SlaveInterface::dispatch()
 void SlaveInterface::calcSpeed()
 {
     Q_D(SlaveInterface);
-    if (d->slave_calcs_speed) {
+    if (d->slave_calcs_speed || !d->connection->isConnected()) { // killing a job results in disconnection but the timer never stops
         d->speed_timer.stop();
         return;
     }
@@ -281,7 +277,7 @@ bool SlaveInterface::dispatch(int _cmd, const QByteArray &rawdata)
     case INF_META_DATA: {
         MetaData m;
         stream >> m;
-        if (m.contains(QStringLiteral("ssl_in_use"))) {
+        if (m.contains(QLatin1String("ssl_in_use"))) {
             const QLatin1String ssl_("ssl_");
             const MetaData constM = m;
             for (MetaData::ConstIterator it = constM.lowerBound(ssl_); it != constM.constEnd(); ++it) {
