@@ -17,15 +17,15 @@
    Boston, MA 02110-1301, USA.
 */
 
-#define QT_NO_CAST_FROM_ASCII
-
 #include "kio_trash.h"
 #include "kiotrashdebug.h"
+#include "../../pathhelpers_p.h"
+
 #include <kio/job.h>
 #include <kio/jobuidelegateextension.h>
 #include <KLocalizedString>
+#include <KDirNotify>
 
-#include <QDebug>
 #include <QMimeDatabase>
 #include <QMimeType>
 #include <QCoreApplication>
@@ -61,6 +61,12 @@ extern "C" {
     }
 }
 
+static bool isTopLevelEntry(const QUrl &url)
+{
+    const QString dir = url.adjusted(QUrl::RemoveFilename).path();
+    return dir.length() <= 1;
+}
+
 #define INIT_IMPL \
     if ( !impl.init() ) { \
         error( impl.lastErrorCode(), impl.lastErrorMessage() ); \
@@ -87,8 +93,8 @@ TrashProtocol::~TrashProtocol()
 void TrashProtocol::enterLoop()
 {
     QEventLoop eventLoop;
-    connect(this, SIGNAL(leaveModality()),
-            &eventLoop, SLOT(quit()));
+    connect(this, &TrashProtocol::leaveModality,
+            &eventLoop, &QEventLoop::quit);
     eventLoop.exec(QEventLoop::ExcludeUserInputEvents);
 }
 
@@ -109,23 +115,21 @@ void TrashProtocol::restore(const QUrl &trashURL)
     }
     QUrl dest = QUrl::fromLocalFile(info.origPath);
     if (!relativePath.isEmpty()) {
-        dest = dest.adjusted(QUrl::StripTrailingSlash);
-        // (u.path() + '/' + txt) '/' is unable to concate ?
-        dest.setPath(dest.path() + QLatin1String("/") + relativePath);
+        dest.setPath(concatPaths(dest.path(), relativePath));
     }
 
     // Check that the destination directory exists, to improve the error code in case it doesn't.
     const QString destDir = dest.adjusted(QUrl::RemoveFilename).path();
     QT_STATBUF buff;
 
-    if (QT_LSTAT(QFile::encodeName(destDir), &buff) == -1) {
+    if (QT_LSTAT(QFile::encodeName(destDir).constData(), &buff) == -1) {
         error(KIO::ERR_SLAVE_DEFINED,
               i18n("The directory %1 does not exist anymore, so it is not possible to restore this item to its original location. "
                    "You can either recreate that directory and use the restore operation again, or drag the item anywhere else to restore it.", destDir));
         return;
     }
 
-    copyOrMove(trashURL, dest, false /*overwrite*/, Move);
+    copyOrMoveFromTrash(trashURL, dest, false /*overwrite*/, Move);
 }
 
 void TrashProtocol::rename(const QUrl &oldURL, const QUrl &newURL, KIO::JobFlags flags)
@@ -135,11 +139,46 @@ void TrashProtocol::rename(const QUrl &oldURL, const QUrl &newURL, KIO::JobFlags
     qCDebug(KIO_TRASH) << "TrashProtocol::rename(): old=" << oldURL << " new=" << newURL << " overwrite=" << (flags & KIO::Overwrite);
 
     if (oldURL.scheme() == QLatin1String("trash") && newURL.scheme() == QLatin1String("trash")) {
-        error(KIO::ERR_CANNOT_RENAME, oldURL.toString());
+        if (!isTopLevelEntry(oldURL) || !isTopLevelEntry(newURL)) {
+            error(KIO::ERR_CANNOT_RENAME, oldURL.toString());
+            return;
+        }
+        int oldTrashId;
+        QString oldFileId, oldRelativePath;
+        bool oldOk = TrashImpl::parseURL(oldURL, oldTrashId, oldFileId, oldRelativePath);
+        if (!oldOk) {
+            error(KIO::ERR_SLAVE_DEFINED, i18n("Malformed URL %1", oldURL.toString()));
+            return;
+        }
+        if (!oldRelativePath.isEmpty()) {
+            error(KIO::ERR_CANNOT_RENAME, oldURL.toString());
+            return;
+        }
+        // Dolphin/KIO can't specify a trashid in the new URL so here path == filename
+        //bool newOk = TrashImpl::parseURL(newURL, newTrashId, newFileId, newRelativePath);
+        const QString newFileId = newURL.path().mid(1);
+        if (newFileId.contains(QLatin1Char('/'))) {
+            error(KIO::ERR_CANNOT_RENAME, oldURL.toString());
+            return;
+        }
+        bool ok = impl.moveInTrash(oldTrashId, oldFileId, newFileId);
+        if (!ok) {
+            error(impl.lastErrorCode(), impl.lastErrorMessage());
+            return;
+        }
+        const QUrl finalUrl = TrashImpl::makeURL(oldTrashId, newFileId, QString());
+        org::kde::KDirNotify::emitFileRenamed(oldURL, finalUrl);
+        finished();
         return;
     }
 
-    copyOrMove(oldURL, newURL, (flags & KIO::Overwrite), Move);
+    if (oldURL.scheme() == QLatin1String("trash") && newURL.isLocalFile()) {
+        copyOrMoveFromTrash(oldURL, newURL, (flags & KIO::Overwrite), Move);
+    } else if (oldURL.isLocalFile() && newURL.scheme() == QLatin1String("trash")) {
+        copyOrMoveToTrash(oldURL, newURL, Move);
+    } else {
+        error(KIO::ERR_UNSUPPORTED_ACTION, i18n("Invalid combination of protocols."));
+    }
 }
 
 void TrashProtocol::copy(const QUrl &src, const QUrl &dest, int /*permissions*/, KIO::JobFlags flags)
@@ -153,104 +192,107 @@ void TrashProtocol::copy(const QUrl &src, const QUrl &dest, int /*permissions*/,
         return;
     }
 
-    copyOrMove(src, dest, (flags & KIO::Overwrite), Copy);
+    if (src.scheme() == QLatin1String("trash") && dest.isLocalFile()) {
+        copyOrMoveFromTrash(src, dest, (flags & KIO::Overwrite), Copy);
+    } else if (src.isLocalFile() && dest.scheme() == QLatin1String("trash")) {
+        copyOrMoveToTrash(src, dest, Copy);
+    } else {
+        error(KIO::ERR_UNSUPPORTED_ACTION, i18n("Invalid combination of protocols."));
+    }
 }
 
-void TrashProtocol::copyOrMove(const QUrl &src, const QUrl &dest, bool overwrite, CopyOrMove action)
+void TrashProtocol::copyOrMoveFromTrash(const QUrl &src, const QUrl &dest, bool overwrite, CopyOrMove action)
 {
-    if (src.scheme() == QLatin1String("trash") && dest.isLocalFile()) {
-        // Extracting (e.g. via dnd). Ignore original location stored in info file.
-        int trashId;
-        QString fileId, relativePath;
-        bool ok = TrashImpl::parseURL(src, trashId, fileId, relativePath);
-        if (!ok) {
-            error(KIO::ERR_SLAVE_DEFINED, i18n("Malformed URL %1", src.toString()));
+    // Extracting (e.g. via dnd). Ignore original location stored in info file.
+    int trashId;
+    QString fileId, relativePath;
+    bool ok = TrashImpl::parseURL(src, trashId, fileId, relativePath);
+    if (!ok) {
+        error(KIO::ERR_SLAVE_DEFINED, i18n("Malformed URL %1", src.toString()));
+        return;
+    }
+    const QString destPath = dest.path();
+    if (QFile::exists(destPath)) {
+        if (overwrite) {
+            ok = QFile::remove(destPath);
+            Q_ASSERT(ok);   // ### TODO
+        } else {
+            error(KIO::ERR_FILE_ALREADY_EXIST, destPath);
             return;
         }
-        const QString destPath = dest.path();
-        if (QFile::exists(destPath)) {
-            if (overwrite) {
-                ok = QFile::remove(destPath);
-                Q_ASSERT(ok);   // ### TODO
-            } else {
-                error(KIO::ERR_FILE_ALREADY_EXIST, destPath);
-                return;
-            }
-        }
+    }
 
-        if (action == Move) {
-            qCDebug(KIO_TRASH) << "calling moveFromTrash(" << destPath << " " << trashId << " " << fileId << ")";
-            ok = impl.moveFromTrash(destPath, trashId, fileId, relativePath);
-        } else { // Copy
-            qCDebug(KIO_TRASH) << "calling copyFromTrash(" << destPath << " " << trashId << " " << fileId << ")";
-            ok = impl.copyFromTrash(destPath, trashId, fileId, relativePath);
+    if (action == Move) {
+        qCDebug(KIO_TRASH) << "calling moveFromTrash(" << destPath << " " << trashId << " " << fileId << ")";
+        ok = impl.moveFromTrash(destPath, trashId, fileId, relativePath);
+    } else { // Copy
+        qCDebug(KIO_TRASH) << "calling copyFromTrash(" << destPath << " " << trashId << " " << fileId << ")";
+        ok = impl.copyFromTrash(destPath, trashId, fileId, relativePath);
+    }
+    if (!ok) {
+        error(impl.lastErrorCode(), impl.lastErrorMessage());
+    } else {
+        if (action == Move && relativePath.isEmpty()) {
+            (void)impl.deleteInfo(trashId, fileId);
         }
-        if (!ok) {
+        finished();
+    }
+}
+
+void TrashProtocol::copyOrMoveToTrash(const QUrl &src, const QUrl &dest, CopyOrMove action)
+{
+    qCDebug(KIO_TRASH) << "trashing a file" << src << dest;
+
+    // Trashing a file
+    // We detect the case where this isn't normal trashing, but
+    // e.g. if kwrite tries to save (moving tempfile over destination)
+    if (isTopLevelEntry(dest) && src.fileName() == dest.fileName()) { // new toplevel entry
+        const QString srcPath = src.path();
+        // In theory we should use TrashImpl::parseURL to give the right filename to createInfo,
+        // in case the trash URL didn't contain the same filename as srcPath.
+        // But this can only happen with copyAs/moveAs, not available in the GUI
+        // for the trash (New/... or Rename from iconview/listview).
+        int trashId;
+        QString fileId;
+        if (!impl.createInfo(srcPath, trashId, fileId)) {
             error(impl.lastErrorCode(), impl.lastErrorMessage());
         } else {
-            if (action == Move && relativePath.isEmpty()) {
-                (void)impl.deleteInfo(trashId, fileId);
+            bool ok;
+            if (action == Move) {
+                qCDebug(KIO_TRASH) << "calling moveToTrash(" << srcPath << " " << trashId << " " << fileId << ")";
+                ok = impl.moveToTrash(srcPath, trashId, fileId);
+            } else { // Copy
+                qCDebug(KIO_TRASH) << "calling copyToTrash(" << srcPath << " " << trashId << " " << fileId << ")";
+                ok = impl.copyToTrash(srcPath, trashId, fileId);
             }
-            finished();
-        }
-        return;
-    } else if (src.isLocalFile() && dest.scheme() == QLatin1String("trash")) {
-        QString dir = dest.adjusted(QUrl::RemoveFilename).path();
-        qCDebug(KIO_TRASH) << "trashing a file to " << dir << src << dest;
-
-        // Trashing a file
-        // We detect the case where this isn't normal trashing, but
-        // e.g. if kwrite tries to save (moving tempfile over destination)
-        if (dir.length() <= 1 && src.fileName() == dest.fileName()) { // new toplevel entry
-            const QString srcPath = src.path();
-            // In theory we should use TrashImpl::parseURL to give the right filename to createInfo,
-            // in case the trash URL didn't contain the same filename as srcPath.
-            // But this can only happen with copyAs/moveAs, not available in the GUI
-            // for the trash (New/... or Rename from iconview/listview).
-            int trashId;
-            QString fileId;
-            if (!impl.createInfo(srcPath, trashId, fileId)) {
+            if (!ok) {
+                (void)impl.deleteInfo(trashId, fileId);
                 error(impl.lastErrorCode(), impl.lastErrorMessage());
             } else {
-                bool ok;
-                if (action == Move) {
-                    qCDebug(KIO_TRASH) << "calling moveToTrash(" << srcPath << " " << trashId << " " << fileId << ")";
-                    ok = impl.moveToTrash(srcPath, trashId, fileId);
-                } else { // Copy
-                    qCDebug(KIO_TRASH) << "calling copyToTrash(" << srcPath << " " << trashId << " " << fileId << ")";
-                    ok = impl.copyToTrash(srcPath, trashId, fileId);
-                }
-                if (!ok) {
-                    (void)impl.deleteInfo(trashId, fileId);
-                    error(impl.lastErrorCode(), impl.lastErrorMessage());
-                } else {
-                    // Inform caller of the final URL. Used by konq_undo.
-                    const QUrl url = impl.makeURL(trashId, fileId, QString());
-                    setMetaData(QLatin1String("trashURL-") + srcPath, url.url());
-                    finished();
-                }
+                // Inform caller of the final URL. Used by konq_undo.
+                const QUrl url = impl.makeURL(trashId, fileId, QString());
+                setMetaData(QLatin1String("trashURL-") + srcPath, url.url());
+                finished();
             }
-            return;
-        } else {
-            qCDebug(KIO_TRASH) << "returning KIO::ERR_ACCESS_DENIED, it's not allowed to add a file to an existing trash directory";
-            // It's not allowed to add a file to an existing trash directory.
-            error(KIO::ERR_ACCESS_DENIED, dest.toString());
-            return;
         }
     } else {
-        error(KIO::ERR_UNSUPPORTED_ACTION, i18n("Internal error in copyOrMove, should never happen"));
+        qCDebug(KIO_TRASH) << "returning KIO::ERR_ACCESS_DENIED, it's not allowed to add a file to an existing trash directory";
+        // It's not allowed to add a file to an existing trash directory.
+        error(KIO::ERR_ACCESS_DENIED, dest.toString());
     }
 }
 
 void TrashProtocol::createTopLevelDirEntry(KIO::UDSEntry &entry)
 {
     entry.clear();
-    entry.insert(KIO::UDSEntry::UDS_NAME, QStringLiteral("."));
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-    entry.insert(KIO::UDSEntry::UDS_ACCESS, 0700);
-    entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
-    entry.insert(KIO::UDSEntry::UDS_USER, m_userName);
-    entry.insert(KIO::UDSEntry::UDS_GROUP, m_groupName);
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("."));
+    entry.fastInsert(KIO::UDSEntry::UDS_DISPLAY_NAME, i18n("Trash"));
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+    entry.fastInsert(KIO::UDSEntry::UDS_ACCESS, 0700);
+    entry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, QStringLiteral("inode/directory"));
+    entry.fastInsert(KIO::UDSEntry::UDS_ICON_NAME, impl.isEmpty() ? QStringLiteral("user-trash") : QStringLiteral("user-trash-full") );
+    entry.fastInsert(KIO::UDSEntry::UDS_USER, m_userName);
+    entry.fastInsert(KIO::UDSEntry::UDS_GROUP, m_groupName);
 }
 
 void TrashProtocol::stat(const QUrl &url)
@@ -368,8 +410,7 @@ void TrashProtocol::listDir(const QUrl &url)
         return;
     }
     if (!relativePath.isEmpty()) {
-        info.physicalPath += QLatin1Char('/');
-        info.physicalPath += relativePath;
+        info.physicalPath += QLatin1Char('/') + relativePath;
     }
 
     // List subdir. Can't use kio_file here since we provide our own info...
@@ -388,8 +429,7 @@ void TrashProtocol::listDir(const QUrl &url)
         //const QString url = TrashImpl::makeURL( trashId, fileId, relativePath + '/' + fileName );
         entry.clear();
         TrashedFileInfo infoForItem(info);
-        infoForItem.origPath += QLatin1Char('/');
-        infoForItem.origPath += fileName;
+        infoForItem.origPath += QLatin1Char('/') + fileName;
         if (createUDSEntry(filePath, fileName, fileName, entry, infoForItem)) {
             listEntry(entry);
         }
@@ -402,18 +442,18 @@ bool TrashProtocol::createUDSEntry(const QString &physicalPath, const QString &d
 {
     QByteArray physicalPath_c = QFile::encodeName(physicalPath);
     QT_STATBUF buff;
-    if (QT_LSTAT(physicalPath_c, &buff) == -1) {
+    if (QT_LSTAT(physicalPath_c.constData(), &buff) == -1) {
         qCWarning(KIO_TRASH) << "couldn't stat " << physicalPath;
         return false;
     }
     if (S_ISLNK(buff.st_mode)) {
         char buffer2[ 1000 ];
-        int n = readlink(physicalPath_c, buffer2, 999);
+        int n = ::readlink(physicalPath_c.constData(), buffer2, 999);
         if (n != -1) {
             buffer2[ n ] = 0;
         }
 
-        entry.insert(KIO::UDSEntry::UDS_LINK_DEST, QFile::decodeName(buffer2));
+        entry.fastInsert(KIO::UDSEntry::UDS_LINK_DEST, QFile::decodeName(buffer2));
         // Follow symlink
         // That makes sense in kio_file, but not in the trash, especially for the size
         // #136876
@@ -432,25 +472,25 @@ bool TrashProtocol::createUDSEntry(const QString &physicalPath, const QString &d
     mode_t access = buff.st_mode & 07777; // extract permissions
     access &= 07555; // make it readonly, since it's in the trashcan
     Q_ASSERT(!internalFileName.isEmpty());
-    entry.insert(KIO::UDSEntry::UDS_NAME, internalFileName);   // internal filename, like "0-foo"
-    entry.insert(KIO::UDSEntry::UDS_DISPLAY_NAME, displayFileName);   // user-visible filename, like "foo"
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, type);
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, internalFileName);   // internal filename, like "0-foo"
+    entry.fastInsert(KIO::UDSEntry::UDS_DISPLAY_NAME, displayFileName);   // user-visible filename, like "foo"
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, type);
     //if ( !url.isEmpty() )
     //    entry.insert( KIO::UDSEntry::UDS_URL, url );
 
     QMimeDatabase db;
     QMimeType mt = db.mimeTypeForFile(physicalPath);
     if (mt.isValid()) {
-        entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, mt.name());
+        entry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, mt.name());
     }
-    entry.insert(KIO::UDSEntry::UDS_ACCESS, access);
-    entry.insert(KIO::UDSEntry::UDS_SIZE, buff.st_size);
-    entry.insert(KIO::UDSEntry::UDS_USER, m_userName);   // assumption
-    entry.insert(KIO::UDSEntry::UDS_GROUP, m_groupName);   // assumption
-    entry.insert(KIO::UDSEntry::UDS_MODIFICATION_TIME, buff.st_mtime);
-    entry.insert(KIO::UDSEntry::UDS_ACCESS_TIME, buff.st_atime);   // ## or use it for deletion time?
-    entry.insert(KIO::UDSEntry::UDS_EXTRA, info.origPath);
-    entry.insert(KIO::UDSEntry::UDS_EXTRA + 1, info.deletionDate.toString(Qt::ISODate));
+    entry.fastInsert(KIO::UDSEntry::UDS_ACCESS, access);
+    entry.fastInsert(KIO::UDSEntry::UDS_SIZE, buff.st_size);
+    entry.fastInsert(KIO::UDSEntry::UDS_USER, m_userName);   // assumption
+    entry.fastInsert(KIO::UDSEntry::UDS_GROUP, m_groupName);   // assumption
+    entry.fastInsert(KIO::UDSEntry::UDS_MODIFICATION_TIME, buff.st_mtime);
+    entry.fastInsert(KIO::UDSEntry::UDS_ACCESS_TIME, buff.st_atime);   // ## or use it for deletion time?
+    entry.fastInsert(KIO::UDSEntry::UDS_EXTRA, info.origPath);
+    entry.fastInsert(KIO::UDSEntry::UDS_EXTRA + 1, info.deletionDate.toString(Qt::ISODate));
     return true;
 }
 
@@ -464,7 +504,6 @@ void TrashProtocol::listRoot()
     listEntry(entry);
     for (TrashedFileInfoList::ConstIterator it = lst.begin(); it != lst.end(); ++it) {
         const QUrl url = TrashImpl::makeURL((*it).trashId, (*it).fileId, QString());
-        QUrl origURL = QUrl::fromLocalFile((*it).origPath);
         entry.clear();
         const QString fileDisplayName = (*it).fileId;
 
@@ -547,13 +586,13 @@ void TrashProtocol::get(const QUrl &url)
     // Usually we run jobs in TrashImpl (for e.g. future kdedmodule)
     // But for this one we wouldn't use DCOP for every bit of data...
     QUrl fileURL = QUrl::fromLocalFile(physicalPath);
-    KIO::Job *job = KIO::get(fileURL, KIO::NoReload, KIO::HideProgressInfo);
-    connect(job, SIGNAL(data(KIO::Job*,QByteArray)),
-            this, SLOT(slotData(KIO::Job*,QByteArray)));
-    connect(job, SIGNAL(mimetype(KIO::Job*,QString)),
-            this, SLOT(slotMimetype(KIO::Job*,QString)));
-    connect(job, SIGNAL(result(KJob*)),
-            this, SLOT(jobFinished(KJob*)));
+    KIO::TransferJob *job = KIO::get(fileURL, KIO::NoReload, KIO::HideProgressInfo);
+    connect(job, &KIO::TransferJob::data,
+            this, &TrashProtocol::slotData);
+    connect(job,  QOverload<KIO::Job*,const QString&>::of(&KIO::TransferJob::mimetype),
+            this, &TrashProtocol::slotMimetype);
+    connect(job, &KJob::result,
+            this, &TrashProtocol::jobFinished);
     enterLoop();
 }
 
