@@ -32,7 +32,6 @@
 #include <kcombobox.h>
 #include <QLineEdit>
 #include <kurllabel.h>
-#include <KConfigGroup>
 #include <QVBoxLayout>
 
 
@@ -46,18 +45,18 @@ public:
         setObjectName (QStringLiteral ("UserAgentSiteNameValidator"));
     }
 
-    State validate (QString& input, int&) const Q_DECL_OVERRIDE
+    State validate (QString& input, int&) const override
     {
         if (input.isEmpty())
             return Intermediate;
 
-        if (input.startsWith (QChar ('.')))
+        if (input.startsWith(QLatin1Char('.')))
             return Invalid;
 
         const int length = input.length();
 
         for (int i = 0 ; i < length; i++) {
-            if (!input[i].isLetterOrNumber() && input[i] != '.' && input[i] != '-')
+            if (!input[i].isLetterOrNumber() && input[i] != QLatin1Char('.') && input[i] != QLatin1Char('-'))
                 return Invalid;
         }
 
@@ -72,16 +71,15 @@ UserAgentSelectorDlg::UserAgentSelectorDlg (UserAgentInfo* info, QWidget* parent
       mButtonBox(nullptr)
 {
     QWidget *mainWidget = new QWidget(this);
-    QVBoxLayout *mainLayout = new QVBoxLayout;
-    setLayout(mainLayout);
+    QVBoxLayout *mainLayout = new QVBoxLayout(this);
     mainLayout->addWidget(mainWidget);
     mUi.setupUi (mainWidget);
 
     mButtonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, this);
     mainLayout->addWidget(mButtonBox);
 
-    connect(mButtonBox, SIGNAL(accepted()), this, SLOT(accept()));
-    connect(mButtonBox, SIGNAL(rejected()), this, SLOT(reject()));
+    connect(mButtonBox, &QDialogButtonBox::accepted, this, &QDialog::accept);
+    connect(mButtonBox, &QDialogButtonBox::rejected, this, &QDialog::reject);
 
     if (!mUserAgentInfo) {
         setEnabled (false);
@@ -98,10 +96,10 @@ UserAgentSelectorDlg::UserAgentSelectorDlg (UserAgentInfo* info, QWidget* parent
     mUi.siteLineEdit->setValidator (validator);
     mUi.siteLineEdit->setFocus();
 
-    connect (mUi.siteLineEdit, SIGNAL (textEdited (QString)),
-             SLOT (onHostNameChanged (QString)));
-    connect (mUi.aliasComboBox, SIGNAL (activated (QString)),
-             SLOT (onAliasChanged (QString)));
+    connect (mUi.siteLineEdit, &QLineEdit::textEdited,
+             this, &UserAgentSelectorDlg::onHostNameChanged);
+    connect (mUi.aliasComboBox, QOverload<const QString &>::of(&QComboBox::activated),
+             this, &UserAgentSelectorDlg::onAliasChanged);
 
     mButtonBox->button(QDialogButtonBox::Ok)->setEnabled (false);
 }
@@ -158,4 +156,4 @@ QString UserAgentSelectorDlg::alias()
     return mUi.identityLineEdit->text();
 }
 
-#include "useragentselectordlg.moc"
\ No newline at end of file
+#include "useragentselectordlg.moc"
